import contextlib
import io
import typing as t
import unittest

from optionresult import Option, Ok, Err

UInt32 = t.NewType("UInt32", int)


def to_uint32(value: int) -> UInt32:
    if value < 0 or value > 2**32 - 1:
        raise ValueError("value out of range")
    return UInt32(value)


class TestOption(unittest.TestCase):
    def test_dunder_eq(self):
        self.assertTrue(Option(2) == Option(2))
        self.assertTrue(Option(None) == Option(None))
        self.assertFalse(Option(2) == 2)

    def test_dunder_repr(self):
        self.assertEqual(repr(Option(2)), "Some(2)")
        self.assertEqual(repr(Option("foo")), "Some('foo')")
        self.assertEqual(repr(Option(None)), "None")

    def test_is_some(self):
        self.assertTrue(Option(2).is_some())
        self.assertFalse(Option(None).is_some())

    def test_is_some_and(self):
        self.assertTrue(Option(2).is_some_and(lambda x: x > 0))
        self.assertFalse(Option(2).is_some_and(lambda x: x < 0))
        self.assertFalse(Option(None).is_some_and(lambda x: x > 0))
        self.assertFalse(Option(None).is_some_and(lambda x: x < 0))

    def test_is_none(self):
        self.assertFalse(Option(2).is_none())
        self.assertTrue(Option(None).is_none())

    def test_expect(self):
        self.assertEqual(Option("value").expect("error"), "value")
        with self.assertRaises(ValueError) as context:
            Option(None).expect("fruits are healthy")
        self.assertEqual("fruits are healthy", str(context.exception))

    def test_unwrap(self):
        self.assertEqual(Option("air").unwrap(), "air")
        with self.assertRaises(RuntimeError) as context:
            Option(None).unwrap()
        self.assertEqual("called `Option.unwrap()` on a `None` value", str(context.exception))

    def test_unwrap_or(self):
        self.assertEqual(Option("car").unwrap_or("bike"), "car")
        self.assertEqual(Option(None).unwrap_or("bike"), "bike")

    def test_unwrap_or_else(self):
        k = 10
        self.assertEqual(Option(4).unwrap_or_else(lambda: 2 * k), 4)
        self.assertEqual(Option(None).unwrap_or_else(lambda: 2 * k), 20)

    def test_map(self):
        self.assertEqual(Option("Hello, World!").map(lambda x: len(x)), Option(13))
        self.assertEqual(Option(None).map(lambda x: len(x)), Option(None))

    def test_inspect(self):
        v = [1, 2, 3, 4, 5]

        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            Option.of(lambda: v[3]).inspect(lambda x: print(f"got: {x}"))
        self.assertEqual(f.getvalue(), "got: 4\n")

        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            Option.of(lambda: v[5]).inspect(lambda x: print(f"got: {x}"))
        self.assertEqual(f.getvalue(), "")

    def test_map_or(self):
        self.assertEqual(Option("foo").map_or(42, len), 3)
        self.assertEqual(Option(None).map_or(42, len), 42)

    def test_map_or_else(self):
        k = 21
        self.assertEqual(Option("foo").map_or_else(lambda: 2 * k, len), 3)
        self.assertEqual(Option(None).map_or_else(lambda: 2 * k, len), 42)

    def test_ok_or(self):
        self.assertEqual(Option("foo").ok_or(0), Ok("foo"))
        self.assertEqual(Option(None).ok_or(0), Err(0))

    def test_ok_or_else(self):
        self.assertEqual(Option("foo").ok_or_else(lambda: 0), Ok("foo"))
        self.assertEqual(Option(None).ok_or_else(lambda: 0), Err(0))

    def test_and_(self):
        self.assertEqual(Option(2).and_(Option(None)), Option(None))
        self.assertEqual(Option(None).and_(Option("foo")), Option(None))
        self.assertEqual(Option(2).and_(Option("foo")), Option("foo"))
        self.assertEqual(Option(None).and_(Option(None)), Option(None))

        self.assertEqual(Option(2) & Option(None), Option(None))
        self.assertEqual(Option(None) & Option("foo"), Option(None))
        self.assertEqual(Option(2) & Option("foo"), Option("foo"))
        self.assertEqual(Option(None) & Option(None), Option(None))

    def test_and_then(self):
        def sq_then_to_string(x: UInt32) -> Option[str]:
            return Option.of(lambda: to_uint32(x * x)).map(str)

        self.assertEqual(Option(to_uint32(2)).and_then(sq_then_to_string), Option("4"))
        self.assertEqual(Option(to_uint32(1_000_000)).and_then(sq_then_to_string), Option(None))
        self.assertEqual(Option(None).and_then(sq_then_to_string), Option(None))

        arr_2d = [["A0", "A1"], ["B0", "B1"]]
        item_0_1 = Option.of(lambda: arr_2d[0]).and_then(lambda row: Option.of(lambda: row[1]))
        self.assertEqual(item_0_1, Option("A1"))
        item_1_0 = Option.of(lambda: arr_2d[2]).and_then(lambda row: Option.of(lambda: row[0]))
        self.assertEqual(item_1_0, Option(None))

    def test_filter(self):
        def is_even(x: int) -> bool:
            return x % 2 == 0

        self.assertEqual(Option(None).filter(is_even), Option(None))
        self.assertEqual(Option(3).filter(is_even), Option(None))
        self.assertEqual(Option(4).filter(is_even), Option(4))

    def test_or_(self):
        self.assertEqual(Option(2).or_(Option(None)), Option(2))
        self.assertEqual(Option(None).or_(Option(100)), Option(100))
        self.assertEqual(Option(2).or_(Option(100)), Option(2))
        self.assertEqual(Option(None).or_(Option(None)), Option(None))

        self.assertEqual(Option(2) | Option(None), Option(2))
        self.assertEqual(Option(None) | Option(100), Option(100))
        self.assertEqual(Option(2) | Option(100), Option(2))
        self.assertEqual(Option(None) | Option(None), Option(None))

    def test_or_else(self):
        def nobody() -> Option[str]:
            return Option(None)

        def vikings() -> Option[str]:
            return Option("vikings")

        self.assertEqual(Option("barbarians").or_else(vikings), Option("barbarians"))
        self.assertEqual(Option(None).or_else(vikings), Option("vikings"))
        self.assertEqual(Option(None).or_else(nobody), Option(None))

    def test_xor(self):
        self.assertEqual(Option(2).xor(Option(None)), Option(2))
        self.assertEqual(Option(None).xor(Option(2)), Option(2))
        self.assertEqual(Option(2).xor(Option(2)), Option(None))
        self.assertEqual(Option(None).xor(Option(None)), Option(None))

        self.assertEqual(Option(2) ^ Option(None), Option(2))
        self.assertEqual(Option(None) ^ Option(2), Option(2))
        self.assertEqual(Option(2) ^ Option(2), Option(None))
        self.assertEqual(Option(None) ^ Option(None), Option(None))



import typing_extensions as t
from optionresult import Err, Ok, Option, Result

UInt32 = t.NewType("UInt32", int)

t.assert_type(Result.of(lambda: 123), t.Union[Ok[int, Exception], Err[int, Exception]])
t.assert_type(Result.of(lambda: None), t.Union[Ok[None, Exception], Err[None, Exception]])

t.assert_type(Ok(-3), Ok[int, t.Any])
t.assert_type(Err("error"), Err[t.Any, str])


t.assert_type(
    Ok[int, int](2)
    .or_else(
        lambda x: Ok[int, int](x * x),
    )
    .or_else(
        lambda x: Ok[int, int](x * x),
    ),
    Ok[int, int],
)
t.assert_type(
    Ok[int, int](2)
    .or_else(
        lambda x: Err[int, int](x * x),
    )
    .or_else(
        lambda x: Ok[int, int](x * x),
    ),
    Ok[int, int],
)
t.assert_type(
    Err[int, int](3)
    .or_else(
        lambda x: Ok[int, int](x * x),
    )
    .or_else(
        lambda x: Err[int, int](x * x),
    ),
    Ok[int, int],
)
t.assert_type(
    Err[int, int](3)
    .or_else(
        lambda x: Err[int, int](x * x),
    )
    .or_else(
        lambda x: Err[int, int](x * x),
    ),
    Err[int, int],
)


t.assert_type(Option.of(lambda: None), Option[None])
t.assert_type(Option.of(lambda: 42), Option[int])


line = "1\n2\n3\n4\na\nb\nc\nd\n"
for num in line.splitlines():
    r = Result[int, Exception].of(lambda: int(num)).map(lambda i: i * 2)
    if isinstance(r, Ok):
        print(r.unwrap())
    elif isinstance(r, Err):
        pass
    else:
        t.assert_never(r)  # type: ignore



from __future__ import annotations

import typing as t

from .exceptions import PanicError

if t.TYPE_CHECKING:
    from .result import Err, Ok # pragma: no cover

T = t.TypeVar("T")
U = t.TypeVar("U")
E = t.TypeVar("E")
R = t.TypeVar("R")
F = t.TypeVar("F")


class Option(t.Generic[T]):
    @staticmethod
    def of(
        fn: t.Callable[..., U],
        *args: t.Any,
        catch: t.Type[Exception] = Exception,
        **kwargs: t.Any,
    ) -> Option[U]:
        try:
            return Option(fn(*args, **kwargs))
        except catch:
            return Option(None)

    def __init__(self, value: T | None):
        self.value = value

    def is_some(self) -> bool:
        return self.value is not None

    def is_some_and(self, f: t.Callable[[T], bool]) -> bool:
        return self.value is not None and f(self.value)

    def is_none(self) -> bool:
        return self.value is None

    def expect(self, msg: str) -> T:
        if self.value is None:
            raise PanicError(msg)
        else:
            return self.value

    def unwrap(self) -> T:
        if self.value is None:
            raise RuntimeError("called `Option.unwrap()` on a `None` value")
        else:
            return self.value

    def unwrap_or(self, default: T) -> T:
        if self.value is None:
            return default
        else:
            return self.value

    def unwrap_or_else(self, f: t.Callable[[], T]) -> T:
        if self.value is None:
            return f()
        else:
            return self.value

    def map(self, f: t.Callable[[T], U]) -> Option[U]:
        if self.value is None:
            return Option(None)
        else:
            return Option(f(self.value))

    def inspect(self, f: t.Callable[[T], None]) -> Option[T]:
        if self.value is not None:
            f(self.value)
        return self

    def map_or(self, default: U, f: t.Callable[[T], U]) -> U:
        if self.value is None:
            return default
        else:
            return f(self.value)

    def map_or_else(self, default: t.Callable[[], U], f: t.Callable[[T], U]) -> U:
        if self.value is None:
            return default()
        else:
            return f(self.value)

    def ok_or(self, err: E) -> Ok[T, E] | Err[T, E]:
        from .result import Err, Ok

        if self.value is None:
            return Err(err)
        else:
            return Ok(self.value)

    def ok_or_else(self, err: t.Callable[[], E]) -> Ok[T, E] | Err[T, E]:
        from .result import Err, Ok

        if self.value is None:
            return Err(err())
        else:
            return Ok(self.value)

    def and_(self, optb: Option[U]) -> Option[U]:
        if self.value is None:
            return Option(None)
        else:
            return optb

    def __and__(self, optb: Option[U]) -> Option[U]:
        return self.and_(optb)

    def and_then(self, f: t.Callable[[T], Option[U]]) -> Option[U]:
        if self.value is None:
            return Option(None)
        else:
            return f(self.value)

    def filter(self, predicate: t.Callable[[T], bool]) -> Option[T]:
        if self.value is None:
            return Option(None)
        if predicate(self.value):
            return Option(self.value)
        else:
            return Option(None)

    def or_(self, optb: Option[T]) -> Option[T]:
        if self.value is None:
            return optb
        else:
            return self

    def __or__(self, optb: Option[T]) -> Option[T]:
        return self.or_(optb)

    def or_else(self, f: t.Callable[[], Option[T]]) -> Option[T]:
        if self.value is None:
            return f()
        else:
            return self

    def xor(self, optb: Option[T]) -> Option[T]:
        if self.value is None and optb.value is not None:
            return optb
        if self.value is not None and optb.value is None:
            return self
        return Option(None)

    def __xor__(self, optb: Option[T]) -> Option[T]:
        return self.xor(optb)

    def __eq__(self, value: object) -> bool:
        if isinstance(value, Option):
            return self.value == value.value
        return False

    def __repr__(self) -> str:
        if self.value is None:
            return "None"
        return f"Some({self.value!r})"



import contextlib
import io
import typing as t
import unittest

from optionresult import Err, Ok, Option, Result, PanicError

UInt32 = t.NewType("UInt32", int)


def to_uint32(value: int) -> UInt32:
    if value < 0 or value > 2**32 - 1:
        raise ValueError("value out of range")
    return UInt32(value)


class TestResult(unittest.TestCase):
    def test_dunder(self):
        self.assertEqual(Ok(2), Ok(2))
        self.assertEqual(Err("error"), Err("error"))
        self.assertNotEqual(Ok(2), 2)
        self.assertNotEqual(Err("error"), Ok(2))
        self.assertNotEqual(Err("error1"), Err("error2"))
        self.assertEqual(repr(Ok(2)), "Ok(2)")
        self.assertEqual(repr(Err("error")), "Err('error')")

    def test_of(self):
        self.assertEqual(Result.of(lambda: 123), Ok(123))

        def raise_error():
            raise ValueError("error")

        x = Result.of(raise_error)
        self.assertEqual(x, Err(ValueError("error")))

    def test_is_ok(self):
        self.assertTrue(Ok(-3).is_ok())
        self.assertFalse(Err("Some error message").is_ok())

    def test_is_ok_and(self):
        self.assertTrue(Ok(2).is_ok_and(lambda x: x > 1))
        self.assertFalse(Ok(0).is_ok_and(lambda x: x > 1))
        self.assertFalse(Err("hey").is_ok_and(lambda x: x > 1))

    def test_is_err(self):
        self.assertFalse(Ok(-3).is_err())
        self.assertTrue(Err("Some error message").is_err())

    def test_is_err_and(self):
        class NotFoundError(Exception):
            pass

        class PermissionDeniedError(Exception):
            pass

        self.assertTrue(Err(NotFoundError("!")).is_err_and(lambda x: isinstance(x, NotFoundError)))
        self.assertFalse(Err(PermissionDeniedError("!")).is_err_and(lambda x: isinstance(x, NotFoundError)))
        self.assertFalse(Ok(123).is_err_and(lambda x: isinstance(x, NotFoundError)))

    def test_ok(self):
        self.assertEqual(Ok(2).ok(), Option(2))
        self.assertEqual(Err("Nothing here").ok(), Option(None))

    def test_err(self):
        self.assertEqual(Ok(2).err(), Option(None))
        self.assertEqual(Err("Nothing here").err(), Option("Nothing here"))

    def test_map(self):
        f = io.StringIO()
        line = "1\n2\n3\n4\na\nb\nc\nd\n"
        with contextlib.redirect_stdout(f):
            for num in line.splitlines():
                r = Result[int, Exception].of(lambda: int(num)).map(lambda i: i * 2)
                if isinstance(r, Ok):
                    print(r.unwrap())
                elif isinstance(r, Err):
                    pass
                else:
                    assert False, "unreachable"  # pragma: no cover
        self.assertEqual(f.getvalue(), "2\n4\n6\n8\n")

    def test_map_or(self):
        self.assertEqual(Ok("foo").map_or(42, len), 3)
        self.assertEqual(Err("bar").map_or(42, len), 42)

    def test_map_or_else(self):
        k = 21
        self.assertEqual(Ok("foo").map_or_else(lambda e: k * 2, len), 3)
        self.assertEqual(Err("bar").map_or_else(lambda e: k * 2, len), 42)

    def test_map_err(self):
        def stringify(x: int) -> str:
            return f"error code: {x}"

        self.assertEqual(Ok(2).map_err(stringify), Ok(2))
        self.assertEqual(Err(13).map_err(stringify), Err("error code: 13"))

    def test_inspect(self):
        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            x = (
                Result.of(lambda: int("4"))
                .inspect(lambda x: print(f"original: {x}"))
                .map(lambda x: x**3)
                .expect("failed to parse number")
            )
        self.assertEqual(x, 64)
        self.assertEqual(f.getvalue(), "original: 4\n")

        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            Err(1).inspect(lambda x: print(f"error: {x}"))
        self.assertEqual(f.getvalue(), "")

    def test_inspect_err(self):
        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            Result.of(lambda: open("address.txt", "r")).inspect_err(lambda e: print(f"failed to read file: {e}"))
        self.assertEqual(f.getvalue(), "failed to read file: [Errno 2] No such file or directory: 'address.txt'\n")

        f = io.StringIO()
        with contextlib.redirect_stdout(f):
            Ok(2).inspect_err(lambda e: print(f"failed to read file: {e}"))
        self.assertEqual(f.getvalue(), "")

    def test_expect(self):
        with self.assertRaises(PanicError) as context:
            Err("emergency failure").expect("Testing expect")
        self.assertEqual("Testing expect: emergency failure", str(context.exception))

    def test_unwrap(self):
        self.assertEqual(Ok(2).unwrap(), 2)
        with self.assertRaises(PanicError) as context:
            Err("emergency failure").unwrap()
        self.assertEqual("emergency failure", str(context.exception))

    def test_expect_err(self):
        with self.assertRaises(PanicError) as context:
            Ok(10).expect_err("Testing expect_err")
        self.assertEqual("Testing expect_err: 10", str(context.exception))

        self.assertEqual(Err(10).expect_err("Testing expect_err"), 10)

    def test_unwrap_err(self):
        with self.assertRaises(PanicError) as context:
            Ok(2).unwrap_err()
        self.assertEqual("2", str(context.exception))
        self.assertEqual(Err("emergency failure").unwrap_err(), "emergency failure")

    def test_and_(self):
        self.assertEqual(Ok(2).and_(Err("late error")), Err("late error"))
        self.assertEqual(Err("early error").and_(Ok("foo")), Err("early error"))
        self.assertEqual(Err("not a 2").and_(Err("late error")), Err("not a 2"))
        self.assertEqual(Ok(2).and_(Ok("different result type")), Ok("different result type"))

        self.assertEqual(Ok(2) & Err("late error"), Err("late error"))
        self.assertEqual(Err("early error") & Ok("foo"), Err("early error"))
        self.assertEqual(Err("not a 2") & Err("late error"), Err("not a 2"))
        self.assertEqual(Ok(2) & Ok("different result type"), Ok("different result type"))

    def test_and_then(self):
        def sq_then_to_string(x: UInt32) -> Result[str, str]:
            return Option.of(lambda: to_uint32(x * x)).map(str).ok_or("overflowed")

        self.assertEqual(Ok(to_uint32(2)).and_then(sq_then_to_string), Ok("4"))
        self.assertEqual(Ok(to_uint32(1_000_000)).and_then(sq_then_to_string), Err("overflowed"))
        self.assertEqual(Err("not a number").and_then(sq_then_to_string), Err("not a number"))

    def test_or_(self):
        self.assertEqual(Ok(2).or_(Err("late error")), Ok(2))
        self.assertEqual(Err("early error").or_(Ok(2)), Ok(2))
        self.assertEqual(Err("not a 2").or_(Err("late error")), Err("late error"))
        self.assertEqual(Ok(2).or_(Ok(100)), Ok(2))

        self.assertEqual(Ok(2) | Err("late error"), Ok(2))
        self.assertEqual(Err("early error") | Ok(2), Ok(2))
        self.assertEqual(Err("not a 2") | Err("late error"), Err("late error"))
        self.assertEqual(Ok(2) | Ok(100), Ok(2))

    def test_or_else(self):
        def sq(x: int) -> Result[int, int]:
            return Ok(x * x)

        def err(x: int) -> Result[int, int]:
            return Err(x)

        self.assertEqual(Ok(2).or_else(sq).or_else(sq), Ok(2))
        self.assertEqual(Ok(2).or_else(err).or_else(sq), Ok(2))
        self.assertEqual(Err(3).or_else(sq).or_else(err), Ok(9))
        self.assertEqual(Err(3).or_else(err).or_else(err), Err(3))

    def test_unwrap_or(self):
        self.assertEqual(Ok(9).unwrap_or(2), 9)
        self.assertEqual(Err("error").unwrap_or(2), 2)

    def test_unwrap_or_else(self):
        self.assertEqual(Ok(2).unwrap_or_else(len), 2)
        self.assertEqual(Err("foo").unwrap_or_else(len), 3)



from __future__ import annotations
import typing as t
from .exceptions import PanicError

if t.TYPE_CHECKING:
    from .option import Option  # pragma: no cover

T = t.TypeVar("T")
U = t.TypeVar("U")
E = t.TypeVar("E")
R = t.TypeVar("R")
F = t.TypeVar("F")


def is_same_exception(exc1: Exception, exc2: Exception) -> bool:
    return type(exc1) == type(exc2) and exc1.args == exc2.args


class Result(t.Generic[T, E]):
    __match_args__ = ("value",)

    @staticmethod
    def of(
        fn: t.Callable[..., U],
        *args: t.Any,
        catch: t.Type[F] = Exception,
        **kwargs: t.Any,
    ) -> Ok[U, F] | Err[U, F]:
        try:
            return Ok(fn(*args, **kwargs))
        except catch as exc:
            return Err(exc)

    def __init__(self, value: T | E):
        raise NotImplementedError

    def is_ok(self) -> bool:
        raise NotImplementedError

    def is_ok_and(self, f: t.Callable[[T], bool]) -> bool:
        raise NotImplementedError

    def is_err(self) -> bool:
        raise NotImplementedError

    def is_err_and(self, f: t.Callable[[E], bool]) -> bool:
        raise NotImplementedError

    def ok(self) -> Option[T]:
        raise NotImplementedError

    def err(self) -> Option[E]:
        raise NotImplementedError

    def map(self, f: t.Callable[[T], U]) -> Ok[U, E] | Err[T, E]:
        raise NotImplementedError

    def map_or(self, default: U, f: t.Callable[[T], U]) -> U:
        raise NotImplementedError

    def map_or_else(self, default: t.Callable[[E], U], f: t.Callable[[T], U]) -> U:
        raise NotImplementedError

    def map_err(self, op: t.Callable[[E], F]) -> Ok[T, E] | Err[T, F]:
        raise NotImplementedError

    def inspect(self, f: t.Callable[[T], None]) -> Ok[T, E] | Err[T, E]:
        raise NotImplementedError

    def inspect_err(self, f: t.Callable[[E], None]) -> Ok[T, E] | Err[T, E]:
        raise NotImplementedError

    def expect(self, msg: str) -> T | t.NoReturn:
        raise NotImplementedError

    def unwrap(self) -> T | t.NoReturn:
        raise NotImplementedError

    def expect_err(self, msg: str) -> t.NoReturn | E:
        raise NotImplementedError

    def unwrap_err(self) -> t.NoReturn | E:
        raise NotImplementedError

    def and_(self, resb: Ok | Err | Result) -> Ok | Err | Result:
        raise NotImplementedError

    @t.overload
    def and_then(self, f: t.Callable[[T], Result[U, F]]) -> Result[U, F]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Ok[U, F]]) -> Ok[U, F]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Err[U, F]]) -> Err[U, F]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Result[U, F]]) -> Result[T, E]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Ok[U, F]]) -> Err[T, E]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Err[U, F]]) -> Err[T, E]: ...

    def and_then(self, f: t.Callable[[T], Ok | Err | Result]) -> Ok | Err | Result:
        raise NotImplementedError

    def or_(self, res: Ok | Err | Result) -> Ok | Err | Result:
        raise NotImplementedError

    def __or__(self, res: Ok | Err | Result) -> Ok | Err | Result:
        raise NotImplementedError

    def or_else(self, op: t.Callable[[E], Ok | Err | Result]) -> Ok | Err | Result:
        raise NotImplementedError

    def unwrap_or(self, default: T) -> T:
        raise NotImplementedError

    def unwrap_or_else(self, f: t.Callable[[E], T]) -> T:
        raise NotImplementedError

    def __eq__(self, other: object) -> bool:
        raise NotImplementedError

    def __repr__(self) -> str:
        raise NotImplementedError


class Ok(Result[T, E]):
    def __init__(self, value: T) -> None:
        self.value = value

    def is_ok(self) -> bool:
        return True

    def is_ok_and(self, f: t.Callable[[T], bool]) -> bool:
        return f(self.unwrap())

    def is_err(self) -> bool:
        return False

    def is_err_and(self, f: t.Callable[[E], bool]) -> bool:
        return self.is_err() and f(self.unwrap_err())

    def ok(self) -> Option[T]:
        from .option import Option

        return Option(self.unwrap())

    def err(self) -> Option[E]:
        from .option import Option

        return Option(None)

    def map(self, f: t.Callable[[T], U]) -> Ok[U, E]:
        return Ok(f(self.unwrap()))

    def map_or(self, default: U, f: t.Callable[[T], U]) -> U:
        return f(self.unwrap())

    def map_or_else(self, default: t.Callable[[E], U], f: t.Callable[[T], U]) -> U:
        return f(self.unwrap())

    def map_err(self, op: t.Callable[[E], F]) -> Ok[T, E]:
        return self

    def inspect(self, f: t.Callable[[T], None]) -> Ok[T, E]:
        f(self.unwrap())
        return self

    def inspect_err(self, f: t.Callable[[E], None]) -> Ok[T, E]:
        return self

    def expect(self, msg: str) -> T:
        return self.unwrap()

    def unwrap(self) -> T:
        return self.value

    def expect_err(self, msg: str) -> t.NoReturn:
        raise PanicError(f"{msg}: {self.unwrap()}")

    def unwrap_err(self) -> t.NoReturn:
        raise PanicError(self.unwrap())

    def and_(self, resb: Ok[T, F] | Err[T, F] | Ok[U, F] | Err[U, F]) -> Ok[T, F] | Err[T, F] | Ok[U, F] | Err[U, F]:
        return resb

    def __and__(self, resb: Ok[T, F] | Err[T, F] | Ok[U, F] | Err[U, F]) -> Ok[T, F] | Err[T, F] | Ok[U, F] | Err[U, F]:
        return self.and_(resb)

    @t.overload
    def and_then(self, f: t.Callable[[T], Result[U, F]]) -> Result[U, F]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Ok[U, F]]) -> Ok[U, F]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Err[U, F]]) -> Err[U, F]: ...

    def and_then(self, f: t.Callable[[T], Ok | Err | Result]):
        return f(self.unwrap())

    def or_(self, res: Ok[T, F] | Err[T, F]) -> Ok[T, E]:
        return self

    def __or__(self, res: Ok[T, F] | Err[T, F]) -> Ok[T, E]:
        return self.or_(res)

    def or_else(self, op: t.Callable[[E], Ok[U, F] | Err[U, F] | Result[U, F]]) -> Ok[T, E]:
        return self

    def unwrap_or(self, default: T) -> T:
        return self.unwrap()

    def unwrap_or_else(self, f: t.Callable[[E], T]) -> T:
        return self.unwrap()

    def __eq__(self, other: object) -> bool:
        if isinstance(other, Ok):
            return self.value == other.value
        return False

    def __repr__(self) -> str:
        return f"Ok({self.value!r})"


class Err(Result[T, E]):
    def __init__(self, value: E) -> None:
        self.value = value

    def is_ok(self) -> bool:
        return False

    def is_ok_and(self, f: t.Callable[[T], bool]) -> bool:
        return False

    def is_err(self) -> bool:
        return True

    def is_err_and(self, f: t.Callable[[E], bool]) -> bool:
        return f(self.unwrap_err())

    def ok(self) -> Option[T]:
        from .option import Option

        return Option(None)

    def err(self) -> Option[E]:
        from .option import Option

        return Option(self.unwrap_err())

    def map(self, f: t.Callable[[T], U]) -> Err[T, E]:
        return self

    def map_or(self, default: U, f: t.Callable[[T], U]) -> U:
        return default

    def map_or_else(self, default: t.Callable[[E], U], f: t.Callable[[T], U]) -> U:
        return default(self.unwrap_err())

    def map_err(self, op: t.Callable[[E], F]) -> Err[T, F]:
        return Err[T, F](op(self.unwrap_err()))

    def inspect(self, f: t.Callable[[T], None]) -> Err[T, E]:
        return self

    def inspect_err(self, f: t.Callable[[E], None]) -> Err[T, E]:
        f(self.unwrap_err())
        return self

    def expect(self, msg: str) -> t.NoReturn:
        raise PanicError(f"{msg}: {self.unwrap_err()}")

    def unwrap(self) -> t.NoReturn:
        raise PanicError(self.unwrap_err())

    def expect_err(self, msg: str) -> E:
        return self.unwrap_err()

    def unwrap_err(self) -> E:
        return self.value

    def and_(self, resb: Ok[T, F] | Err[T, F]) -> Err[T, E]:
        return self

    def __and__(self, resb: Ok[T, F] | Err[T, F]) -> Err[T, E]:
        return self.and_(resb)

    @t.overload
    def and_then(self, f: t.Callable[[T], Result[U, F]]) -> Err[T, E]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Ok[U, F]]) -> Err[T, E]: ...
    @t.overload
    def and_then(self, f: t.Callable[[T], Err[U, F]]) -> Err[T, E]: ...

    def and_then(self, f: t.Callable[[T], Ok | Err | Result]) -> Err[T, E]:
        return self

    def or_(self, res: "Ok[T, F] | Err[T, F]") -> Ok[T, F] | Err[T, F]:
        return res

    def __or__(self, res: "Ok[T, F] | Err[T, F]") -> Ok[T, F] | Err[T, F]:
        return self.or_(res)

    @t.overload
    def or_else(self, op: t.Callable[[E], Ok[U, F]]) -> Ok[U, F]: ...
    @t.overload
    def or_else(self, op: t.Callable[[E], Err[U, F]]) -> Err[U, F]: ...
    @t.overload
    def or_else(self, op: t.Callable[[E], Result[U, F]]) -> Result[U, F]: ...

    def or_else(self, op: t.Callable[[E], Ok[U, F] | Err[U, F] | Result[U, F]]) -> Ok[U, F] | Err[U, F] | Result[U, F]:
        return op(self.unwrap_err())

    def unwrap_or(self, default: T) -> T:
        return default

    def unwrap_or_else(self, f: t.Callable[[E], T]) -> T:
        return f(self.unwrap_err())

    def __eq__(self, value: object) -> bool:
        if isinstance(value, Err):
            if isinstance(self.value, Exception) and isinstance(value.value, Exception):
                return is_same_exception(self.value, value.value)
            return self.value == value.value
        return False

    def __repr__(self) -> str:
        return f"Err({self.value!r})"
